

#include "as32m0_i2c.h"
#define I2C_OSC_CLK  16
#define TX_FIFO_DEPTH 8
#define RX_FIFO_DEPTH 8

void I2C_DISABLE(I2C_TypeDef *I2C,uint32_t data)
{
	I2C->IC_ENABLE &= ~(1<<data);
}
void I2C_ENABLE(I2C_TypeDef *I2C,uint32_t data)
{
	I2C->IC_ENABLE |= (1<<data);
}
void I2C_TargerAdd(I2C_TypeDef *I2C,uint32_t data)
{
	I2C->IC_TAR = data;
}
void I2C_SlaveAdd(I2C_TypeDef *I2C,uint32_t data)
{
	I2C->IC_SAR = data;
}
void I2C_SCL_HCNT(I2C_TypeDef *I2C,uint8_t speed,uint8_t fastmodeplus)
{
  uint32_t MIN_SCL_HighTime=0;
  float temp;
  if(speed == 0x01 )
  {
     MIN_SCL_HighTime=4000;//ns
     temp=I2C_OSC_CLK*MIN_SCL_HighTime*0.001;
     I2C->IC_SS_SCL_HCNT=(uint32_t)temp;
  }
  else if(speed == 0x02)
  {
    if(fastmodeplus == 0x01)
     MIN_SCL_HighTime=260;//ns
    else
     MIN_SCL_HighTime=600;//ns
      
     temp=I2C_OSC_CLK*MIN_SCL_HighTime*0.001;
     I2C->IC_FS_SCL_HCNT=(uint32_t)temp;
  }
  else if(speed == 0x03)
  {
     MIN_SCL_HighTime=120;//ns
     temp=I2C_OSC_CLK*MIN_SCL_HighTime*0.001;
  //   I2C->IC_HS_SCL_HCNT=(uint32_t)temp;
     I2C->IC_FS_SCL_HCNT=0;
     I2C->IC_HS_SCL_HCNT=0;
     I2C->IC_HS_SPKLEN = 2;
     I2C->IC_FS_SPKLEN = 2;
  }
  else
    printf("ERROR:I2C_SCL_HCNT is not exist !");

}

void I2C_SCL_LCNT(I2C_TypeDef *I2C,uint8_t speed,uint8_t fastmodeplus)
{
  uint32_t MIN_SCL_LowTime=0;
  float temp;
  if(speed == 0x01 )
  {   
     MIN_SCL_LowTime=4700;//ns
     temp=I2C_OSC_CLK*MIN_SCL_LowTime*0.001;
     I2C->IC_SS_SCL_LCNT=(uint32_t)temp;
  }
  else if(speed == 0x02)
  { 
    if(fastmodeplus == 0x01)
     MIN_SCL_LowTime=500;//ns
    else
      MIN_SCL_LowTime=1300;//ns
     temp=I2C_OSC_CLK*MIN_SCL_LowTime*0.001;
     I2C->IC_FS_SCL_LCNT=(uint32_t)temp;
  }
  else if(speed == 0x03)
  {
     MIN_SCL_LowTime=320;//ns
     temp=I2C_OSC_CLK*MIN_SCL_LowTime*0.001;
    // I2C->IC_HS_SCL_LCNT=(uint32_t)temp;	
     I2C->IC_FS_SCL_LCNT=0;
     I2C->IC_HS_SCL_LCNT=0;
  }
  else
   printf("ERROR:I2C_SCL_LCNT is not exist !");
}
void I2C_TransFifoSet(I2C_TypeDef *I2C,uint8_t data)
{
   if(data < (TX_FIFO_DEPTH+1))
    I2C->IC_TX_TL=data;
   else
      printf("ERROR:The IC_TX_TL must be less than %x \n ",TX_FIFO_DEPTH);	
}
void I2C_ReceFifoSet(I2C_TypeDef *I2C,uint8_t data)
{
   if(data < (RX_FIFO_DEPTH+1))
    I2C->IC_RX_TL=data;
   else
      printf("ERROR:The IC_TX_TL must be less than %x \n ",TX_FIFO_DEPTH);	
}
void I2C_MasterConfig(I2C_TypeDef *I2C,
                        uint8_t TarAddress,
                        uint8_t IcSlaveDis,
                        uint8_t Speed,
                        uint8_t MasterMode,
                        uint8_t tx_fifo,
                        uint8_t rx_fifo,
                        uint8_t fastmodeplus
			)
{
     I2C_DISABLE(I2C,bwI2C_ENAble); 
     I2C->IC_CON     &= ~(0xffff<< 0);//clear bit[15:0]
     I2C->IC_CON      = (IcSlaveDis<<6)|(Speed<<1)|(MasterMode<<0)|(1<<5)|(1<<8);
     APB_I2C->IC_INTR_MASK = 0x00;
     I2C_TargerAdd(I2C,TarAddress);
     I2C_SCL_LCNT(I2C,Speed,fastmodeplus);
     I2C_SCL_HCNT(I2C,Speed,fastmodeplus);
     I2C_TransFifoSet(I2C,tx_fifo);
     I2C_ReceFifoSet(I2C,tx_fifo);
     I2C_ENABLE(I2C,bwI2C_ENAble);
}
void I2C_SlaveConfig(I2C_TypeDef *I2C,
                        uint8_t SlaAddress,
                        uint8_t Speed,
                        uint8_t tx_fifo,
                        uint8_t rx_fifo,
                        uint8_t fastmodeplus
			)
{
     I2C_DISABLE(I2C,bwI2C_ENAble);
     I2C->IC_CON     &= ~(0xffff<< 0);//clear bit[15:0]
     I2C->IC_CON  = (Speed<<1);
     I2C_SlaveAdd(I2C,SlaAddress);
     I2C_SCL_LCNT(I2C,Speed,fastmodeplus);
     I2C_SCL_HCNT(I2C,Speed,fastmodeplus);
     I2C_TransFifoSet(I2C,tx_fifo);
     I2C_ReceFifoSet(I2C,tx_fifo);
 	 I2C->IC_INTR_MASK &= ~(0x7fff);
 	 I2C->IC_INTR_MASK |= 1<<11;
 	 I2C->IC_INTR_MASK |= 1<<5;//REQ Interrupt
     I2C_ENABLE(I2C,bwI2C_ENAble);
}



void I2CWirteOneByte(I2C_TypeDef *I2C,uint8_t OffsetAddr,uint8_t data)
{
      int i;
      I2C->IC_DATA_CMD =OffsetAddr;//set offset address
      I2C->IC_DATA_CMD =data;//set offset address
      for(i=0;i<500000;i++);
}
uint8_t I2CReadOneByte(I2C_TypeDef *I2C,uint8_t OffsetAddr)
{
      int i;
      I2C->IC_DATA_CMD =OffsetAddr;//set offset address
      I2C->IC_DATA_CMD =0x100;//send read command
      for(i=0;i<500000;i++);
      return I2C->IC_DATA_CMD;
}
void I2CWirteOneByte_Loopback(I2C_TypeDef *S_I2C,uint8_t data)
{
	S_I2C->IC_DATA_CMD =data;
}

void I2CWirteBytes(I2C_TypeDef *I2C,uint8_t *data,uint8_t len){
	uint8_t i;
	while((APB_I2C->IC_STATUS & (1<<0)) == 0x1);
	for(i=0; i<len; i++, data++){
	 I2C->IC_DATA_CMD = *data;
	}
	while((APB_I2C->IC_STATUS & (1<<2)) == 0x0);
}




